# noqa: D205,D208,D400
"""
    formelsammlung
    ~~~~~~~~~~~~~~

    Collection of different multipurpose functions.

    :copyright: (c) Christian Riedel
    :license: GPLv3, see LICENSE for more details
"""
try:
    from importlib.metadata import version
except ModuleNotFoundError:
    from importlib_metadata import version  # type: ignore[import,no-redef]

__version__ = version(__name__)



# noqa: D205,D208,D400
"""
    formelsammlung.flask_sphinx_docs
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Serve sphinx docs in your flask app.

    :copyright: (c) Christian Riedel
    :license: GPLv3
"""
from pathlib import Path
from typing import Optional

from flask import Flask, Response


class SphinxDocServer:  # pylint: disable=R0903
    """Serve your sphinx docs under `/docs/` on your own flask app.

    .. highlight:: python

    You can invoke it in your app factory::

        sds = SphinxDocServer()

        def create_app():
            app = Flask(__name__)
            sds.init_app(app)
            return app

    or you can include the plugin directly without setting a ``doc_dir``::

        app = Flask(__name__)
        SphinxDocServer(app)

    or with setting a ``doc_dir``::

        app = Flask(__name__)
        SphinxDocServer(app, doc_dir="../../docs/build/html")

    .. highlight:: default
    """

    def __init__(self, app: Optional[Flask] = None, **kwargs) -> None:
        """Init SphinxDocServer."""
        if app is not None:
            self.init_app(app, **kwargs)

    def init_app(
        self, app: Flask, doc_dir: Optional[str] = None, index_file: str = "index.html"
    ) -> None:
        """Add the `/docs/` route to the `app` object.

        :param app: Flask object to add the route to.
        :param doc_dir: The base directory holding the sphinx docs to serve. If not set
            the ``doc_dir`` is guessed up to 3 directories above.
        :param index_file: The html file containing the base toctree.
            Default: "index.html"
        """

        @app.route("/docs/", defaults={"filename": index_file})
        @app.route("/docs/<path:filename>")
        def web_docs(filename: str) -> Response:  # pylint: disable=W0612
            """Route the given doc page.

            :param filename: File name from URL
            :return: Requested doc page
            """
            app.static_folder = doc_dir or self._find_build_docs(app.root_path or "")
            doc_file = app.send_static_file(filename)
            app.static_folder = "static"
            return doc_file

    @staticmethod
    def _find_build_docs(app_root: str, steps_up_the_tree: int = 3):
        """Find build sphinx html docs.

        :param app_root: Root directory of the app.
        :param steps_up_the_tree: Amount of steps to go up the file tree, defaults to 3
        :raises IOError: if no root dir path for the app is given.
        :raises IOError: if no 'doc' or 'docs' directory is found.
        :raises IOError: if no '_build' or 'build' directory is found in the
            doc/docs dir.
        :raises IOError: if no 'html' directory is found in the _build/build dir.
        :return: Path to directory holding the build sphinx docs.
        """
        if not app_root:
            raise OSError("Got no root dir for the flask app to start search.")

        check_dir = file_dir = Path(app_root).parent

        #: Search doc(s) dir up the tree
        doc_dir = None
        for i in range(0, steps_up_the_tree + 1):
            if (check_dir / "doc").is_dir():
                doc_dir = check_dir / "doc"
            if (check_dir / "docs").is_dir():
                doc_dir = check_dir / "docs"

            if doc_dir:
                break

            check_dir = file_dir.parents[i]

        if not doc_dir:
            raise OSError("No 'doc' or 'docs' directory found.")

        #: search for (_)build dir
        build_dir = None
        if (doc_dir / "_build").is_dir():
            build_dir = doc_dir / "_build"
        if (doc_dir / "build").is_dir():
            build_dir = doc_dir / "build"

        if not build_dir:
            raise OSError(
                f"No '_build' or 'build' directory found in {doc_dir}. "
                "Maybe you forgot to build the docs."
            )

        #: check for html dir
        if (build_dir / "html").is_dir():
            return build_dir / "html"
        raise OSError(
            f"No 'html' directory found in {build_dir}. "
            "Maybe you forgot to build the HTML docs."
        )



# noqa: D205,D208,D400
"""
    formelsammlung.strcalc
    ~~~~~~~~~~~~~~~~~~~~~~

    Calculate arithmetic expressions from strings.

    :copyright: (c) Christian Riedel
    :license: GPLv3
"""
import ast
import operator

from typing import Optional, Union


NumberType = Union[int, float, complex]


class _StringCalculator(ast.NodeVisitor):
    """Calculate an arithmetic expression from a string using :mod:`ast`."""

    # pylint: disable=C0103,R0201
    def visit_BinOp(self, node: ast.BinOp) -> NumberType:  # noqa: N802
        """Handle `BinOp` nodes."""
        return {
            ast.Add: operator.add,  #: a + b
            ast.Sub: operator.sub,  #: a - b
            ast.Mult: operator.mul,  #: a * b
            ast.Pow: operator.pow,  #: a ** b
            ast.Div: operator.truediv,  #: a / b
            ast.FloorDiv: operator.floordiv,  #: a // b
            ast.Mod: operator.mod,  #: a % b
        }[type(node.op)](self.visit(node.left), self.visit(node.right))

    # fmt: off
    def visit_UnaryOp(self, node: ast.UnaryOp) -> NumberType:  # noqa: N802
        """Handle `UnaryOp` nodes."""
        return {
            ast.UAdd: operator.pos,  #: + a
            ast.USub: operator.neg,  #: - a
        }[type(node.op)](self.visit(node.operand))
    # fmt: on

    def visit_Constant(self, node: ast.Constant) -> NumberType:  # noqa: N802
        """Handle `Constant` nodes."""
        return node.value

    def visit_Num(self, node: ast.Num) -> NumberType:  # noqa: N802
        """Handle `Num` nodes.

        For backwards compatibility <3.8. Since 3.8 ``visit_Constant`` is used.
        """
        return node.n

    def visit_Expr(self, node: ast.Expr) -> NumberType:  # noqa: N802
        """Handle `Expr` nodes."""
        return self.visit(node.value)


def calculate_string(expression: str) -> Optional[NumberType]:
    """Calculate the given expression.

    The given arithmetic expression string is parsed as an :mod:`ast` and then
    handled by the :class:`ast.NodeVisitor`.

    Python exceptions are risen like with normal arithmetic expression e.g.
    :class:`ZeroDivisionError`.

    Supported number types:

        - :class:`int` ``1``
        - :class:`float` ``1.1``
        - :class:`complex` ``1+1j``

    .. warning::
        On PyPy3 only:
        When working with :class:`complex` numbers containing or resulting with
        :class:`float` numbers be aware that the result of :func:`calculate_string` and
        the equivalent arithmetic expression can divert in the decimals. The result from
        :func:`calculate_string` is then less precise.

    Supported mathematical operators:

        - Positive (:func:`operator.pos`) ``+ a``
        - Negative (:func:`operator.neg`) ``- a``
        - Addition (:func:`operator.add`) ``a + b``
        - Subtraction (:func:`operator.sub`) ``a - b``
        - Multiplication (:func:`operator.mul`) ``a * b``
        - Exponentiation (:func:`operator.pow`) ``a ** b``
        - Division (:func:`operator.truediv`) ``a / b``
        - FloorDivision (:func:`operator.floordiv`) ``a // b``
        - Modulo (:func:`operator.mod`) ``a % b``

    How to use:

    .. testsetup::

        from formelsammlung.strcalc import calculate_string

    .. doctest::

        >>> calculate_string("(1+2)/3")
        1.0

    :param expression: String with arithmetic expression.
    :return: Result or None
    """
    if expression == "":
        return None
    return _StringCalculator().visit(ast.parse(expression).body[0])



# noqa: D205,D208,D400
"""
    formelsammlung.envvar
    ~~~~~~~~~~~~~~~~~~~~~

    Get environment variables and transform their type.

    :copyright: (c) Christian Riedel
    :license: GPLv3
"""
import os
import re

from typing import Any, Iterable, Optional


TRUE_BOOL_VALUES = ("1", "y", "yes", "t", "True")
FALSE_BOOL_VALUES = ("0", "n", "no", "f", "False")


def getenv_typed(
    var_name: str,
    default: Any = None,
    rv_type: Optional[type] = None,
    *,
    raise_error_if_no_value: bool = False,
    true_bool_values: Optional[Iterable] = None,
    false_bool_values: Optional[Iterable] = None,
) -> Any:
    """Wrap :func:`os.getenv` to adjust the type of the values.

    Instead of returning the environments variable's value as :class:`str` like
    :func:`os.getenv` you can set ``rv_type`` to a type to convert the value to. If
    ``rv_type`` is not set the type gets guessed and used for conversion.

    Guessable types are:

        - :class:`bool`
        - :class:`int`
        - :class:`float`
        - :class:`str` (fallback)

    How to use:

    .. testsetup::

        import os
        from formelsammlung.envvar import getenv_typed

    .. doctest::

        >>> os.environ["TEST_ENV_VAR"] = "2"
        >>> getenv_typed("TEST_ENV_VAR", 1, int)
        2

    .. testcleanup::

        os.environ["TEST_ENV_VAR"] = ""

    :param var_name: Name of the environment variable.
    :param default: Default value if no value is found for ``var_name``.
        Default: ``None``.
    :param rv_type: Type the value of the environment variable should be changed into.
        If not set or set to ``None`` the type gets guessed.
        Default: ``None``.
    :param raise_error_if_no_value: If ``True`` raises an :class:`KeyError` when no
        value is found for ``var_name`` and ``default`` is ``None``.
        Parameter is keyword only.
        Default: ``False``
    :param true_bool_values: Set of objects whose string representations are
        matched case-insensitive against the environment variable's value if the
        ``rv_type`` is :class:`bool` or the type gets guessed. If a match is found
        ``True`` is returned.
        Parameter is keyword only.
        Default: ``(1, "y", "yes", "t", True)``
    :param false_bool_values: Set of objects whose string representations are
        matched case-insensitive against the environment variable's value if the
        ``rv_type`` is :class:`bool` or the type gets guessed. If a match is found
        ``False`` is returned.
        Parameter is keyword only.
        Default: ``(0, "n", "no", "f", False)``
    :raises KeyError: If ``raise_error_if_no_value`` is ``True`` and no value is found
        for ``var_name`` and ``default`` is ``None``.
    :raises KeyError: If ``rv_type`` is set to :class:`bool` and value from
        ``var_name`` or ``default`` is not found in ``true_bool_values`` or
        ``false_bool_values``.
    :return: Value for ``var_name`` or ``default`` converted to ``rv_type``
        or guessed type.
    """
    env_var = os.getenv(var_name, default)

    if not env_var and default is None:
        if raise_error_if_no_value:
            raise KeyError(
                f"Environment variable '{var_name}' not set or empty and no default."
            ) from None
        return None

    #: Convert to given `rv_type` if set.
    if rv_type and rv_type is not bool:
        return rv_type(env_var)

    env_var = str(env_var)

    #: Guess bool value
    true_bool_values = set(true_bool_values or TRUE_BOOL_VALUES)
    false_bool_values = set(false_bool_values or FALSE_BOOL_VALUES)

    if env_var.casefold() in (str(b).casefold() for b in true_bool_values):
        return True
    if env_var.casefold() in (str(b).casefold() for b in false_bool_values):
        return False
    if rv_type:
        raise KeyError(
            f"Environment variable '{var_name}' has an invalid Boolean value.\n"
            f"For true use any of: {true_bool_values}\n"
            f"For false use any of: {false_bool_values}"
        ) from None

    #: Guess if `int`
    if re.fullmatch(r"^\d+$", env_var):
        env_var = int(env_var)

    #: Guess if `float`
    elif re.fullmatch(r"^\d+\.\d+$", env_var):
        env_var = float(env_var)

    return env_var



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['formelsammlung']

package_data = \
{'': ['*']}

extras_require = \
{':python_version < "3.8"': ['importlib-metadata>1.6'],
 'docs': ['sphinx>=3.2,<4.0',
          'sphinx-rtd-theme>=0.5,<0.6',
          'sphinx-autodoc-typehints>=1.11.0,<2.0.0',
          'sphinxcontrib-apidoc>=0.3.0,<0.4.0'],
 'flask': ['flask>=1.1.2,<2.0.0'],
 'pre-commit': ['pre-commit>=2.7,<3.0', 'mypy==0.782', 'pylint>=2.6.0,<3.0.0'],
 'testing': ['pytest>=6,<7',
             'pytest-xdist>=1.34,<2.0',
             'pytest-cov>=2.10,<3.0',
             'coverage[toml]>=5.2.1,<6.0.0',
             'pytest-sugar>=0.9.4,<0.10.0',
             'pytest-randomly>=3.4,<4.0',
             'pytest-flask>=1.0,<2.0']}

setup_kwargs = {
    'name': 'formelsammlung',
    'version': '0.3.2',
    'description': 'Collection of different functions',
    'long_description': "==============\nformelsammlung\n==============\n\n+---------------+----------------------------------------------------------------------+\n| **General**   | |maintenance| |license| |black| |rtd|                                |\n+---------------+----------------------------------------------------------------------+\n| **Pipeline**  | |azure_pipeline| |azure_coverage|                                    |\n+---------------+----------------------------------------------------------------------+\n| **Tools**     | |poetry| |tox| |pytest| |sphinx|                                     |\n+---------------+----------------------------------------------------------------------+\n| **VC**        | |vcs| |gpg| |semver| |pre-commit|                                    |\n+---------------+----------------------------------------------------------------------+\n| **Github**    | |gh_release| |gh_commits_since| |gh_last_commit|                     |\n|               +----------------------------------------------------------------------+\n|               | |gh_stars| |gh_forks| |gh_contributors| |gh_watchers|                |\n+---------------+----------------------------------------------------------------------+\n| **PyPI**      | |pypi_release| |pypi_py_versions| |pypi_implementations|             |\n|               +----------------------------------------------------------------------+\n|               | |pypi_status| |pypi_format| |pypi_downloads|                         |\n+---------------+----------------------------------------------------------------------+\n\n\n**Collection of different multipurpose functions.**\n\nThis library is a collection of different functions I developed which I use in different\nprojects so I put them here.\n\n\nFunctionality\n=============\n\n- ``getenv_typed()``: is a wrapper around ``os.getenv`` returning the value of the environment variable in the correct python type.\n- ``calculate_string()``: takes an arithmetic expression as a string and calculates it.\n- ``SphinxDocServer``: is a flask plugin to serve the repository's docs build as HTML (by sphinx). Needs ``flask`` extra to be also installed to work.\n\n\nPrerequisites\n=============\n\n*Works only with python version >= 3.6*\n\nA new version of ``pip`` that supports PEP-517/PEP-518 is required.\nWhen the setup fails try updating ``pip``.\n\n\nDisclaimer\n==========\n\nNo active maintenance is intended for this project.\nYou may leave an issue if you have a questions, bug report or feature request,\nbut I cannot promise a quick response time.\n\n\n.. .############################### LINKS ###############################\n\n\n.. General\n.. |maintenance| image:: https://img.shields.io/badge/No%20Maintenance%20Intended-X-red.svg?style=flat-square\n    :target: http://unmaintained.tech/\n    :alt: Maintenance - not intended\n\n.. |license| image:: https://img.shields.io/github/license/Cielquan/formelsammlung.svg?style=flat-square&label=License\n    :alt: License\n    :target: https://github.com/Cielquan/formelsammlung/blob/master/LICENSE.txt\n\n.. |black| image:: https://img.shields.io/badge/Code%20Style-black-000000.svg?style=flat-square\n    :alt: Code Style - Black\n    :target: https://github.com/psf/black\n\n.. |rtd| image:: https://img.shields.io/readthedocs/formelsammlung/latest.svg?style=flat-square&logo=read-the-docs&logoColor=white&label=Read%20the%20Docs\n    :alt: Read the Docs - Build Status (latest)\n    :target: https://formelsammlung.readthedocs.io/en/latest/\n\n\n.. Pipeline\n.. |azure_pipeline| image:: https://img.shields.io/azure-devops/build/cielquan/05507266-5d2e-4862-80f9-9f2b439814c8/8?style=flat-square&logo=azure-pipelines&label=Azure%20Pipelines\n    :target: https://dev.azure.com/cielquan/formelsammlung/_build/latest?definitionId=8&branchName=master\n    :alt: Azure DevOps builds\n\n.. |azure_coverage| image:: https://img.shields.io/azure-devops/coverage/cielquan/formelsammlung/8?style=flat-square&logo=azure-pipelines&label=Coverage\n    :target: https://dev.azure.com/cielquan/formelsammlung/_build/latest?definitionId=8&branchName=master\n    :alt: Azure DevOps Coverage\n\n\n.. Tools\n.. |poetry| image:: https://img.shields.io/badge/Packaging-poetry-brightgreen.svg?style=flat-square\n    :target: https://python-poetry.org/\n    :alt: Poetry\n\n.. |tox| image:: https://img.shields.io/badge/Automation-tox-brightgreen.svg?style=flat-square\n    :target: https://tox.readthedocs.io/en/latest/\n    :alt: tox\n\n.. |pytest| image:: https://img.shields.io/badge/Test%20framework-pytest-brightgreen.svg?style=flat-square\n    :target: https://docs.pytest.org/en/latest/\n    :alt: Pytest\n\n.. |sphinx| image:: https://img.shields.io/badge/Doc%20builder-sphinx-brightgreen.svg?style=flat-square\n    :target: https://www.sphinx-doc.org/\n    :alt: Sphinx\n\n\n.. VC\n.. |vcs| image:: https://img.shields.io/badge/VCS-git-orange.svg?style=flat-square&logo=git\n    :target: https://git-scm.com/\n    :alt: VCS\n\n.. |gpg| image:: https://img.shields.io/badge/GPG-signed-blue.svg?style=flat-square&logo=gnu-privacy-guard\n    :target: https://gnupg.org/\n    :alt: Website\n\n.. |semver| image:: https://img.shields.io/badge/Versioning-semantic-brightgreen.svg?style=flat-square\n    :alt: Versioning - semantic\n    :target: https://semver.org/\n\n.. |pre-commit| image:: https://img.shields.io/badge/pre--commit-enabled-brightgreen?style=flat-square&logo=pre-commit&logoColor=yellow\n    :target: https://github.com/pre-commit/pre-commit\n    :alt: pre-commit\n\n\n.. Github\n.. |gh_release| image:: https://img.shields.io/github/v/release/Cielquan/formelsammlung.svg?style=flat-square&logo=github\n    :alt: Github - Latest Release\n    :target: https://github.com/Cielquan/formelsammlung/releases/latest\n\n.. |gh_commits_since| image:: https://img.shields.io/github/commits-since/Cielquan/formelsammlung/latest.svg?style=flat-square&logo=github\n    :alt: Github - Commits since latest release\n    :target: https://github.com/Cielquan/formelsammlung/commits/master\n\n.. |gh_last_commit| image:: https://img.shields.io/github/last-commit/Cielquan/formelsammlung.svg?style=flat-square&logo=github\n    :alt: Github - Last Commit\n    :target: https://github.com/Cielquan/formelsammlung/commits/master\n\n.. |gh_stars| image:: https://img.shields.io/github/stars/Cielquan/formelsammlung.svg?style=flat-square&logo=github\n    :alt: Github - Stars\n    :target: https://github.com/Cielquan/formelsammlung/stargazers\n\n.. |gh_forks| image:: https://img.shields.io/github/forks/Cielquan/formelsammlung.svg?style=flat-square&logo=github\n    :alt: Github - Forks\n    :target: https://github.com/Cielquan/formelsammlung/network/members\n\n.. |gh_contributors| image:: https://img.shields.io/github/contributors/Cielquan/formelsammlung.svg?style=flat-square&logo=github\n    :alt: Github - Contributors\n    :target: https://github.com/Cielquan/formelsammlung/graphs/contributors\n\n.. |gh_watchers| image:: https://img.shields.io/github/watchers/Cielquan/formelsammlung.svg?style=flat-square&logo=github\n    :alt: Github - Watchers\n    :target: https://github.com/Cielquan/formelsammlung/watchers\n\n\n.. PyPI\n.. |pypi_release| image:: https://img.shields.io/pypi/v/formelsammlung.svg?style=flat-square&logo=pypi&logoColor=FBE072\n    :alt: PyPI - Package latest release\n    :target: https://pypi.org/project/formelsammlung/\n\n.. |pypi_py_versions| image:: https://img.shields.io/pypi/pyversions/formelsammlung.svg?style=flat-square&logo=python&logoColor=FBE072\n    :alt: PyPI - Supported Python Versions\n    :target: https://pypi.org/project/formelsammlung/\n\n.. |pypi_implementations| image:: https://img.shields.io/pypi/implementation/formelsammlung.svg?style=flat-square&logo=python&logoColor=FBE072\n    :alt: PyPI - Supported Implementations\n    :target: https://pypi.org/project/formelsammlung/\n\n.. |pypi_status| image:: https://img.shields.io/pypi/status/formelsammlung.svg?style=flat-square&logo=pypi&logoColor=FBE072\n    :alt: PyPI - Stability\n    :target: https://pypi.org/project/formelsammlung/\n\n.. |pypi_format| image:: https://img.shields.io/pypi/format/formelsammlung.svg?style=flat-square&logo=pypi&logoColor=FBE072\n    :alt: PyPI - Format\n    :target: https://pypi.org/project/formelsammlung/\n\n.. |pypi_downloads| image:: https://img.shields.io/pypi/dm/formelsammlung.svg?style=flat-square&logo=pypi&logoColor=FBE072\n    :target: https://pypi.org/project/formelsammlung/\n    :alt: PyPI - Monthly downloads\n",
    'author': 'Cielquan',
    'author_email': 'cielquan@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Cielquan/formelsammlung',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'extras_require': extras_require,
    'python_requires': '>=3.6.1,<4.0.0',
}


setup(**setup_kwargs)
